/***************************************************************************
 *            basicwindow.h
 *
 *  Fri Jan 21 23:00:12 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef BASICWINDOW_H
#define BASICWINDOW_H

#include <gtk/gtk.h>
#include <iostream>

#include <gdk/gdkkeysyms.h>
#include "imuserdaemon.h"
#include "settings.h"
#include "utilities.h"

/** some container modes - not used yet
*/
enum ContainerModes
{
	CM_TYPE,
	CM_USER,
	CM_MESSAGEWINDOW
};

class WindowContainer;

/** A basic window.
	This is the fundamental class for all windows in icqnd. It abstracts the
	GTK window stuf and allows things such as adding into a container (to have
	many windows in one separated by tabs). Also this window contains a
	rudimentary structure for signal handling and user Settings handling

	Creation of a window is fairly easy:
	<pre>
	BasicWindow *testWindow = new BasicWindow();

	testWindow->setWindowTitle("Some empty window");
	testWindow->setWindowSize(200, 200);
	testWindow->createWindow();
	</pre>
	The window widget itself can be accessed by BasicWindow::content which gets
	filled by the function BasicWindow::createWindowContent (which is called from
	BasicWindow::createWindow() and is not accessible to the outside.)

	If a window wants to be a tab in a WindowContainer the window class must
	implement the function createContainerTabWidget in order to provide an 
	icon and a label for the notebook tab (it is later accessed from).
*/

class BasicWindow
{
	public:
		UserEventType	wType;		/**< the user event the window implements */	
		IMEventManager	*manager;	/**< the event manager for this window	  */
		GtkWidget		*content,	/**< the window content					  */
						*tabLabel;
		gint			sizeX,
						sizeY;
		GtkAccelGroup	*accelerators;
	
		BasicWindow();
		virtual ~BasicWindow();
	
		/** Creates a basic window */
		gboolean createWindow();
		
		/** Called when a window looses focus. Should be overwritten if needed. */
		virtual void windowDeactivated();
		/** Called when a window gets focus. Should be overwritten if needed. */
		virtual void windowActivated();
	
		/** Give the focus to that window. */
		virtual void focusWindow();
	
		/** Set the window title.
			@param _title The new window title
		*/
		void setWindowTitle(gchar *_title);
				
		/** Set the windows size
			@param _sizeX The new width for the window
			@param _sizeY The new height for the window
		*/
		void setWindowSize(gint _sizeX, gint _sizeY);
	
		/** The callback function for the manager
			The callback function to connect to the manager. It is called 
			whenever the manager gets an event or a signal or something.
			Should be overwritten (and not called from other places than the manager)
			
			@param command The command, mostly one of enum NotificationTypes.
			@param lastResult The result of the last operation if there was one
			@param info Additional information for the command.
			@return TRUE if the event was properly processed.
		*/
		virtual gboolean eventCallback(gint command, gint lastResult, gpointer info);
	
		/** The callback function for the Settings manager
			The callback function to connect to the Settings manager. This
			function is called when the Settings values of the category the 
			window is listening to (after calling addSettingsListener) change.
			Should be overwritten and not called from other places than the
			SettingsManager.
			@param sname The name of the section (the category) of Settings
			@param values A GList of SettingsProperty* that contain the name and the new value for a changed user setting
		*/	
		virtual void SettingsChangedCallback(gchar *sname, GList *values);
	
		/** Destroys the window */
		gboolean destroyWindow();
		
		/** Close the window (is called when the window is closed manually) */
		virtual void closeWindow();
		
		/** Add a listener to the Settings.
			This function adds a listener for a specific section (or category)
			of user Settings. Whenever these Settings are changed then the 
			function BasicWindow::SettingsChangedCallback will be called
			@param sname The section to add a listener for
		*/
		virtual void addSettingsListener(gchar *sname);
		
		/** Allocates extra space.
			Allocates extra room in the window on the right side. Used for
			the multi-send part of the conversation window.
			@param toAllocate The width in pixels to increase the windows size
		*/
		virtual void allocateExtraSpace(gint toAllocate);
		
		/** Deallocates the extra space
			Reverts the effect of BasicWindow::allocateExtraSpace.
		*/
		virtual void deallocateExtraSpace();

		/** Sets a callback for destruction
			If you want to listen to the destruction of windows outside of them
			(if you want to listen to them from the window class itself you can
			 overwrite destroyWindowContent) this function makes it possible to
			add a callback function that will be called when the window is 
			destroyed. It takes a function of the type IMEventCallback. When 
			the function is called: 
			- the "command" field will be set to WC_DESTROYED
			- the "type" field will be set to the window type
			- and the data will be set to the window itself
			
			@param _dc The destruction callback
			@param _dcd User data for the call
		*/
		void setDestroyCallback(IMEventCallback _dc, gpointer _dcd);

		/** The static function for the event callback.
			Looks for an updated user status and sets the window icon if
			possible. Then calls BasicWindow::eventCallback. */
		static gboolean cb_eventCallback(UserEventType type, gint command, gint lastResult, gpointer info, BasicWindow *self);

	
	protected:
		
		GtkWidget	*window;
		gchar		*title;
		gboolean	hasFocus;
		gboolean	hasSettingsEntries;
		gint		extraSpace;
	
		IMEventCallback	destroyCallback;
		gpointer		destroyCallbackData;
		
		// creates the real window content
		/** Creates the window content. */
		virtual GtkWidget* createWindowContent();
		/** Destroys the window content. */
		virtual void	   destroyWindowContent();

		/** Gets the container the window is in if this is the case.
			Queries the list of window containers and returns the WindowContainer, 
			this window is in. If the window isn't in any container the function
			returns NULL. 
			@returns A pointer to the window container or NULL */
		gpointer getWindowContainer();
	
		/** Creates a tab widget for the container. 
			The widget created by this function will be shown in the
			notebook tab for this window. Should be overwritten to provide a
			useful container tab widget.
			@returns The widget to use in the notebook tab. */
		virtual GtkWidget* createContainerTabWidget();
	
		/** ESC event handler. 
			This function gets called when the user presses ESC while the window
			has the user focus. The default behavoir is to close the window 
			(by calling BasicWindow::destroyWindow()). Function should be 
			overwritten to stop processing events if there are such. */
		virtual void escPressed();

	
		// callbacks
		/** The static destroy callback.
			This static destruction callback calls BasicWindow::destroyWindow().
			It is connected to the "delete-event" signal of the window. */
		static gboolean cb_destroyWindow(BasicWindow *self);
		/** The static focus-out callback.
			This static focus-out callback calls BasicWindow::windowDeactivated().
			It is connected to the "focus-out-event" signal of the window. */
		static gboolean cb_windowFocusOut(BasicWindow *self);
		/** The static focus-in callback.
			This static focus-in callback calls BasicWindow::windowActivated().
			It is connected to the "focus-in-event" signal of the window. */
		static gboolean cb_windowFocusIn(BasicWindow *self);
		/** The static ESC callback.
			This static ESC callback calls BasicWindow::escPressed(). 
			It is connected by an accelerator (which gets set up in BasicWindow::createWindow(). */
		static void		cb_escPressed(BasicWindow* self);
	
		/** The static Settings changed callback.
			This static Settings-changed callback calls BasicWindow::SettingsChangedCallback().
			It is connected in BasicWindow::createWindow(). */
		static void cb_SettingsChangedCallback(gchar *sname, GList *values, BasicWindow *self);


	friend class WindowContainer;
};



/** A container that holds windows.
	The WindowContainer acts as a container for windows (of class BasicWindow). 
	It arranges them in tabs. The content of each tab is created by
	BasicWindow::createWindowContent() and the tab label is created by 
	BasicWindow::createContainerTabWidget(). The "window" property of the 
	BasicWindow then becomes the one of the container.
	Currently only used for the message window. */
class WindowContainer
{
	public:
		gint 		cType;		 /**< The container type. Not used yet. */
		GtkWidget	*window;	 /**< The GtkWindow. */
		GList		*shortCuts;  /**< A list of shortcuts (will an array of hold u_shortCutType) */
	
		WindowContainer();
		~WindowContainer();
	
		/** Adds a window.
			Adds a window and creates a new notebook page. The tab label and the
			content must me initialized (by BasicWindow::createWindowContent() and
			BasicWindow::createContainerTabWidget()). 
			@param toAdd The window to add
		*/	
		void addWindow(BasicWindow *toAdd);
		/** Removes a window.
			Removes a window from the window container. It doesn't uninitialize
			anything but just gets the window out of the window list and of the
			notebook. 
			@param toRemove the window to remove
			@param the content of the window (normally BasicWindow->content)
		*/
		void removeWindow(BasicWindow *toRemove, GtkWidget *rcontent);
		/** Focus a window.
			Presents one of the windows in the notebook to the user. The window
			itself can notice it because BasicWindow::windowActivated() gets called.
			@param toFocus The window to set the focus on
		*/
		void focusWindow(BasicWindow *toFocus);
		/** Allocates space on the right side of the window. 
			See BasicWindow::allocateExtraSpace().
			@param toAllocate The width in pixels to increase the windows size. */
		void allocateExtraSpace(gint toAllocate);
		/** Deallocates space on the right side of the window.
			See BasicWindow::deallocateExtraSpace(). */
		void deallocateExtraSpace();
		/** Update the window title.
			Updates the window title to the title of the focused child window. */ 
		void updateWindowTitle();
	
		/** Checks whether a window is a child of the container.
			Checks whether a specific window is part of the container. Returns
			TRUE if this is the case.
			@param toLook The window to look for
			@returns TRUE if the window is part of the container
		*/
		gboolean containsWindow(BasicWindow *toLook);
	
	private:
		GtkWidget* noteBook;				/**< The notebook that holds the windows */
		gint	   extraSpace,
				   extraSpaceCount;
	
		GList *children;					/**< A list of children (from type BasicWindow) */
	
		/** Sets up the shortcuts.*/
		void setupAccelerators();
	
		/** The static function for container destruction.
			This function is called when the container shall be destroyed. It
			just deletes the container (where the delete function deletes the
			child windows as well). It is connected to the "delete-event" signal
			of the container window. */
		static void cb_destroyContainer(WindowContainer *self);
		/** The static focus-out function for the container.
			The function looks for the active window and then calls
			windowDeactivated() on it. It is connected to the "focus-out-event" signal
			of the container window. */
		static gboolean cb_containerFocusOut(WindowContainer *self);
		/** The static focus-out function for the container.
			The function looks for the active window and then calls
			windowActivated() on it. It is connected to the "focus-in-event" signal
			of the container window. */
		static gboolean cb_containerFocusIn(WindowContainer *self);
		/** The static switch function for the container.
			This function is called when the user switches a tab. It calls
			windowDeactivated() on the last window, calls windowActivated() on 
			the child window the user switched to and updates the window title.
			It is connected to the "switch-page" signal of the notebook.*/
		static void cb_switchWindow(GtkWidget *w, GtkNotebookPage *page, guint newTab, WindowContainer *self);
		/** The static close-button-clicked function for a window tab.
			The function calls BasicWindow::closeWindow() and then BasicWindow::destroyWindow()
			of the child window the close button is for.*/
		static void cb_closeWindowButtonClicked(GtkWidget *button, WindowContainer *self);
		/** Closes the active tab.
			Never used. */
		static void cb_closeActiveTab(WindowContainer *self);
		/** The static ESC callback.
			The function is called when the usrer presses ESC while the container window has focus.*/
		static void	cb_escPressed(WindowContainer *self);
	
		/** The static Settings-changed callback.
			The function will get connected to the "hotkeys" section of the
			Settings and updates the shortcuts for "Go to the next tab" and
			"Go to the previous tab". */
		static void cb_SettingsChangedCallback(gchar *sname, GList *values, WindowContainer *self);
	
		/** The accelerator function for changing to the previous tab. */
		static void cb_accelLeftPressed(WindowContainer *self);
		/** The accelerator function for changing to the next tab. */
		static void cb_accelRightPressed(WindowContainer *self);
};


/** Get all windows.
	Gets all windows that are currently initialized.
	@returns A GList with the windows. */
GList *BasicWindow_getAllWindows();


#endif
