/***************************************************************************
 *            chatwindow.h
 *
 *  Tue Feb 15 04:08:09 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef _CHATWINDOW_H
#define _CHATWINDOW_H

#include <gtk/gtk.h>
#include "icons.h"
#include "systemcharset.h"
#include "basicwindow.h"
#include "imuserdaemon.h"
#include "processingani.h"
#include "requestdialog.h"
#include "gettext.h"
#include "userutilities.h"

/** A list of different chat view types. See text in ChatWindow for more info.
	The Values are stored in the viewType field of ChatWindowView. */
enum ChatViewTypes
{
	CWVT_PROTO,		/**< Nothing */
	CWVT_WAIT,		/**< A wait view for connections (not used yet) */
	CWVT_REMOTE,	/**< A normal remote view */
	CWVT_OWNER,		/**< The local view */
	CWVT_IRC		/**< The view in IRC style */
};

/** A list of different font sections.
	Used for updating specific sections of font information 
	(in ChatWindowLocalView::updateFontInformation()) */
enum ChatFontTypes
{
	CF_FGCOLOR 		= 1<<0,
	CF_BGCOLOR 		= 1<<1,
	CF_FAMILY		= 1<<2,
	CF_SIZE			= 1<<3,	
	CF_FACE			= 1<<4,
	CF_ALL			= 0xFF
};


class ChatWindow;

/** The interface class for a chat window view. See the intro text of
	ChatWindow for more information.*/
class ChatWindowView
{
	public:
	
		GtkWidget	*textBox,
					*boxFrame,
					*parentBox;
	
		gchar	*alias,
				*userID,
				*charset;

		GString *currentLine,		/**< A buffer for the current line (characters after the last RETURN) */
				*history;			/**< The characters sent by this user so far */
	
		gchar	viewType;			/**< The type of this view (one of ChatViewTypes) */
	
		GtkTextTag 	*fontProperties;
		GtkTreeIter listIter;
	
		gboolean useUserFont,
				 useUserColors;
	
		ChatWindow *parentWindow;	/**< A reference of the ChatWindow that holds this view */
	
	
		/** Initializes the chat view for a specific user.
	
			@param _alias The alias (user name) of the user
			@param _userID The user ID of the user
			@param _parentWindow A reference to the ChatWindow that contains the view
		*/
		ChatWindowView(gchar *_alias, gchar *_userID, ChatWindow *_parentWindow);
		virtual ~ChatWindowView();
	
		/** Creates the widget for the view.
			@returns The widget containing the view. */
		virtual GtkWidget* createChatView();
		/** Destroys the widget for the view. */
		virtual void destroyChatView();
		/** Creates the toolbar for this view.
			@returns A widget containing the toolbar. */	
		virtual GtkWidget* createToolBar();	
		/** Update the font information.
			Updates the font information from the manager and puts it into
			the text view. 
			@param ev The chat event that contains the font information. */
		void updateStyle(ChatEventInfo *ev);
		/** Creates a font context for the text view and stores the current
			font information in it. */
		void createFontContext();
	
	protected:
		GtkWidget	*encodingsMenu;
	
		void		setSelectedEncodingsMenuItem(gchar *id);
	
		virtual void encodingChanged(CharsetType *enc);
	
		
		static void cb_showSubmenu(GtkWidget *subMenu);
		static void cb_selectEncoding(GtkWidget *item, ChatWindowView *self);	
};





/** A chat view for the local user. 
	This is the view where the local user types the messages and selects
	his/her chat Settings. */
class ChatWindowLocalView: public ChatWindowView
{
	public:
		GtkWidget	*ircButton;	
	
		/** Initializes the view.
			@param _parentWindow A reference to the ChatWindow that contains the view */
		ChatWindowLocalView(ChatWindow *_parentWindow);
		virtual ~ChatWindowLocalView();	
	
		/** Creates the toolbar for this view.
			@returns A widget containing the toolbar. */
		virtual GtkWidget* createToolBar();
	
		/** Creates the widget for the view.
			@returns The widget containing the view. */	
		virtual GtkWidget* createChatView();
	
		/** Gets the font properties of the current view and updates the 
			manager with this information. The manager then might send the info
			to the other users.
			@param toUpdate The field to update the info for (see ChatFontTypes). */
		void	updateFontInformation(guint toUpdate);
		/** Clears the text buffer. */
		void	clearTextBuffer();
	
	private:
		guint	currentEncoding;
	
		GtkWidget *createEncodingsMenu();
		GtkWidget *createColorMenus(gboolean createBG);
	
		/** The encoding has changed.
			This function is called when the user changes the user encoding.
			@param enc The new Encoding. */
		void encodingChanged(CharsetType *enc);
	
		/** The callback for sorting the font list.
			It is connected to the fonts list by g_qsort_with_data in 
			ChatWindowLocalView::createToolBar(). */
		static gint cb_sortFamilies(PangoFontFamily **a, PangoFontFamily **b);
	
		/** The static color selection callback.
			This function is called when the user selected a new color in the
			colors list of the toolbar. It is connected to the "activate" signal
			of the list elements. */
		static void cb_selectColor(GtkWidget *item, ChatWindowLocalView *self);
		/** The static font selection callback.
			This function is called when the user selected a new font in the
			fonts list of the toolbar. It is connected to the "activate" signal
			of the list elements. */
		static void cb_selectFont(GtkWidget *box, ChatWindowLocalView *self);
		/** The static size selection callback.
			This function is called when the user selected a new size in the
			sizes list of the toolbar. It is connected to the "activate" signal
			of the list elements. */
		static void cb_selectSize(GtkWidget *box, ChatWindowLocalView *self);
		/** The static bold selection callback.
			This function is called when the user clicks the "Bold" button 
			of the toolbar. It is connected to the "clicked" signal of that button.*/
		static void cb_selectBold(GtkWidget *btn, ChatWindowLocalView *self);
		/** The static italic selection callback.
			This function is called when the user clicks the "Italic" button
			of the toolbar. It is connected to the "clicked" signal of that button. */
		static void cb_selectItalic(GtkWidget *btn, ChatWindowLocalView *self);
		/** The static underline selection callback.
			This function is called when the user clicks the "Underline" button
			of the toolbar. It is connected to the "clicked" signal of that button. */
		static void cb_selectUnderline(GtkWidget *btn, ChatWindowLocalView *self);
		/** The static strikeout selection callback.
			This function is called when the user clicks the "strikeout" button
			of the toolbar. It is connected to the "clicked" signal of that button. */
		static void cb_selectStrikeout(GtkWidget *btn, ChatWindowLocalView *self);
	
		/** The static callback for every key press in the user entry field.
			This function is called whenever the user presses a key. 
			It tries to find out which type of key press it was (character, escape sequence)
			and sends the appropriate information to the chat server. */
		static gboolean cb_keyPressed(GtkWidget *w, GdkEventKey *event, ChatWindowLocalView *self);
		/** The static callback for every text insertation.
			If the user inserted text from the clipboard into the text view
			this function gets called and sends the information character by 
			character. */
		static void	cb_textInserted(GtkTextBuffer *textbuffer, GtkTextIter *arg1, gchar *arg2, gint arg3, ChatWindowLocalView *self);
};


/** A chat window view for remote users.
	This view is for one user only. The user gets an own text entry where
	his messages can be seen. */
class ChatWindowRemoteView: public ChatWindowView
{
	public:	
		/** Initializes the chat view for a specific user.
	
			@param _alias The alias (user name) of the user
			@param _userID The user ID of the user
			@param _parentWindow A reference to the ChatWindow that contains the view
		*/
		ChatWindowRemoteView(gchar *_alias, gchar *_userID, ChatWindow *_parentWindow);
		virtual ~ChatWindowRemoteView();
	
		/** Creates the widget for the view.
			@returns The widget containing the view. */
		GtkWidget *createChatView();
	
		/** Inserts a character of message from the user. 
			@param character The character (letter, escape sequence) to insert. */
		void insertCharacter(gchar *character);
		/** Deletes a character from the end of the message. */
		void deleteCharacter();
	
		/** Returns the content of the current line.
			Returns the content of the current line (the characters after the last RETURN)
			and frees the buffer for this line.
			@returns A char* that contains the line. */
		gchar *getCurrentLineAndClear();
	
		/** Creates the toolbar for this view.
			@returns A widget containing the toolbar. */
		virtual GtkWidget *createToolBar();
	
		/** Sends a kick-user signal to the chat server. Only supported in icq.
			@param alias The alias that shall be kicked
			@param uin The UIN of that user. */
		void	kickUser(gchar *alias, gulong uin);
			
		/** Creates a popup menu for the view. 
			@returns The newly created menu. */
		GtkMenu	*createUserMenu();
	
	private:
		GtkWidget	*fontButtonImg,
					*colButtonImg;
	
		GtkMenu		*userMenu;
	
	/** The static use-font-button-clicked callback.
		This function is called when the user clicks the button to use chat fonts
		(instead of system fonts). It is connected  to the "clicked" signal
		of the "Enable/Diable the user font" button (in the toolbar). */
	static void cb_setUseFont(ChatWindowRemoteView *self);
	/** The static use-colors-button-clicked callback.
		This function is called when the user clicks the button to use chat colors
		(instead of system colors). It is connected to the "clicked" signal 
		of the "Enable/Disable the user colors" button (in the toolbar). */
	static void cb_setUseColors(ChatWindowRemoteView *self);
	/** The static kick-user-button-clicked callback.
		This function is called when the user clicks the button to kick a user.
		It is connected to the "clicked" signal of the "Kick User" button 
		(in the toolbar). */
	static void cb_kickUserButtonClicked(ChatWindowRemoteView *self);
	/** The static menu-item-clicked callback.
		This is the place where selections in the popup menu of that view go to. */
	static void cb_menuItemClicked(GtkMenuItem *item, ChatWindowRemoteView *self);
};



/** A chat window view in IRC style.
	This view differs from the ChatWindowRemoteView in the way the information
	is inserted; only whole lines are added. */
class ChatWindowIRCView: public ChatWindowView
{
	public:
		GtkTextBuffer *textBuffer;		/**< the text buffer for the view */
		GtkTextTag	  *fontComment,		/**< the font info for any information except messages and alias names */
					  *fontAlias;		/**< the font info for the aliases */
	
		ChatWindowIRCView(ChatWindow *_parentWindow);
		virtual ~ChatWindowIRCView();

		/** Creates the view 
			@returns The widget that contains the view. */	
		virtual GtkWidget* createChatView();
	
		/** Inserts a line of chat information.
			@param alias The alias of the user that typed a line
			@param line The line itself
			@param font The font information for the user
			@param isComment Do we really add a line from a user or is it just a status message / comment?
		*/
		void insertLine(gchar* alias, gchar* line, GtkTextTag* font, gboolean isComment);
};



/** The chat window.

	This window represents the multi party chat in icqnd. The window itself
	is, on the visual side, just a container that can hold:
		- a local chat view (class ChatWindowLocalView) which is the area where the local user can type,
		- a list of remote views (class ChatWindowRemoteView) where each represents a user in the chat or
		- a single remote view in IRC style that holds all users in one window.
	A chat view can be created and is then added to the views by calling ChatWindow::addChatView.

	On the internal side the chat window dispatches the information to the views.
	The remote views get their information by insertCharacter or deleteCharacter 
	(which adds or deletes a character). The exception is the IRC view that gets the
	information through ChatWindowIRCView::insertLine().
*/
class ChatWindow: public BasicWindow
{
	public:
		GtkWidget		*mainChatBox,
						*chatVBox,			/**< a vertical box that holds the views */
						*lastFreeCVPlace,	/**< the next place to hold a view */
						*userList,			/**< the users list */
						*userListFrame;		/**< a frame around the users list */
	
		GtkListStore	*usersStore;		/**< the users store accociated with userList */
	
		GList				*chatViews;		/**< a list that contains the view instances */
		ChatWindowLocalView	*ownerView;		/**< the local, ChatWindowLocalView */
		ChatWindowIRCView 	*ircView;		/**< the IRC view */
	
		gboolean		isIRCStyle,			/**< do we use IRC mode? */
						isServer;			/**< did we start the chat session* */
	
		ChatWindow		*multipartyWindow;	/**< if the user selects to invite the user in existing chat room this widget points to it */
	
		ProcessingAni	*connectAni;		/**< the connect animation */
	
		RequestDialog	*request;			/**< the request dialog */


		gint	requestID;
		
		/** The constructor.
			Initializes basic stuff.
			@param _isServer does the window start as a server? */
		ChatWindow(gboolean _isServer);
		~ChatWindow();
	
		/** Get the chat window title.
			Creates the window title for the chat window. This title contains
			either all chat members or some (if there are too much members).
			@returns The newly created title */
		gchar *getChatWindowTitle();
	
		void focusWindow();
		/** Creates the request dialog.
			Creates the request dialog either as a server or as a remote.
			The function creates a new RequestDialog and fills it with the 
			return value of chatWindow::createSpecialWidget(). 
			@param event The chat event that initialized the whole thing */
		void createRequestDialog(ChatEventInfo* event);
	
		/** Rebuilds the chat view.
			After switching from one-area-per-user style to IRC style or after
			a user was added this function must be called to recreate the view. */
		void rebuildViewMode();
		
		/** Not used. */
		void startChat();	
	
		/** Callback for clicking the change-view button.
			This callback is invoked after the user clicked the change-view button.
			It swaps the state of ChatWindow::isIRCStyle and then calls
			rebuildViewMode() to rebuild the view. */
		static void cb_changeViewButtonClicked(ChatWindow* chw);
		/** Callback for toggling the mode to start a chat.
			In the request part of the chat this callback sets the sensitivity
			of the list of existing chat rooms according to the selection the
			user makes ("invite user to an existing chat room" / "start a new chat") */
		static void cb_chatModeSelectionClicked(ChatWindow* chw);

	private:
		GtkWidget	*chatRoomList,
					*startNewChatButton,
					*joinExistingChatButton;	
	
		/** Creates the window content. */
		virtual GtkWidget* createWindowContent();
		/** Destroys the window content. */
		virtual void	   destroyWindowContent();

		/** The event callback for the chat manager. */
		virtual gboolean eventCallback(gint command, gint lastResult, gpointer info);	
	
		/** Creates the button bar.
			@returns The widget which contains the button bar. */
		GtkWidget* createButtonbar();
		/** Creates the users list.
			@returns The widget which contains the users list. */
		GtkWidget* createUserList();
		/** Create a widget for the request dialog. 
			@param canMultiparty Does the remote side allows to be added to an existing chat room?
			@returns The widget which contains the interactive area for the request dialog.*/
		GtkWidget* createSpecialWidget(gboolean canMultiparty);
		/** Finds a multiparty window. 
			
			Finds an existing window if the user selected to add the user into an existing chatroom.
			@returns The ChatWindow or NULL if there is none. */	
		ChatWindow* getMultipartyWindow();
		/** Adds a chat view into the list of current visible views.
			@param view The view that shall be added. */
		void addViewToList(ChatWindowView* view);
		/** Removes a chat view from the list of visible views.
			@param view The view that shall be removed. */
		void removeViewFromList(ChatWindowView* view);
	
		/** Creates a list of users that can be invited.
			This function isn't done yet and a task for the future.
			@returns The widget which contains a list of users. */
		GtkWidget* createUserSelection();
	
		/** Returns the font information the local user is using.
			The result is stored in a new allocated struct from type ChatFontInfo.
			@returns The font information. */
		ChatFontInfo* getFontInfo();
	
		/** Adds a chat view and makes in visible.
			@param cwv The chat view to make visible. */
		void	   addChatView(ChatWindowView* cwv);
	
		/** Finds a chat view by its ID.
			@param id The ID of the user to find the view for
			@returns The chat window view or NULL if none could be found. */
		ChatWindowView* findViewByID(gchar* id);
	
	
		// callbacks	
		/** The request window callback. 
			See the doc for requestWindow for more details. */
		static void cb_requestCallback(gint result, gpointer info, ChatWindow* cw);
		/** The callback for clicking the user list.
			The function is called when the user list is clicked. If the user
			clicks the right mouse button a popup menu will be presented. 
			The function is connected to the "button-press-event" of the users list. */
		static gboolean cb_userListClicked(GtkTreeView* uList, GdkEventButton* event, ChatWindow* cw);		

};



#endif /* _CHATWINDOW_H */
